/*
 * Copyright (C) 2006
 *	Tommy Scheunemann <net@arrishq.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <pwd.h>
#include <limits.h>
#include <strings.h>

#include "wmconfig.h"
#include "prototypes.h"
#include "package.h"

/* define default output dir
 if the directory exists and there are
 contents, they will be replaced with
 Wmconfig entries
*/

/* Default Output Dir */

#define E17_DIR		".e/e/applications/menu/all/wmconfig"

extern const char *root_menu_name;
extern const char *output_directory;
extern unsigned int flags;

/* Enlightenment DR17 output function */

/* Output application launchers */

static void create_desktop_entry(const char *file, struct package *app, const char *use_term)
{
 int fd;
 FILE *f;
 if (app == (struct package *)NULL) {
  return;
 }
 if (app->exec != NULL) {
  char *p;
  p = strrchr(app->exec, '&');
  if (p != NULL)
   *p = '\0';
 }
 if (file == NULL || app->exec == NULL || app->restart)
  return;
#ifdef BSD
 fd = open(file, O_CREAT | O_EXCL | O_NOFOLLOW | O_RDWR, 0600);
#else
 fd = open(file, O_CREAT | O_EXCL | O_RDWR, 0600);
#endif
 f = fdopen (fd, "w+");
 if (f == (FILE *) NULL) {
  fprintf(stderr,"Could not create file %s\n", file);
  perror("Error code");
  return;
 }
 
/* Do not create restart and quit entries - just applications */
 
 if (! app->restart) {
  fprintf(f, "[Desktop Entry]\n");
  if (app->name != NULL)
   fprintf(f, "Name=%s\n", app->name);
  if (app->icon != NULL)
   fprintf(f, "Icon=%s\n", app->icon);
  if (app->description != NULL)
   fprintf(f, "Comment=%s\n", app->description);
  if (app->terminal != NULL)
   fprintf(f, "Exec=%s -e %s\n", use_term, app->exec);
  else
   fprintf(f, "Exec=%s\n", app->exec);
  fprintf(f, "Type=Application\n");
  if (app->mimetype != NULL)
   fprintf(f, "MimeType=%s;\n", app->mimetype);
  fclose(f);
 }
}

/* Create the directories required for the submenus */

static void make_directory_path(const char *path)
{
 char buf[PATH_MAX];
 int pi=0;
 int bi=0;
 while( path[pi] != '\0' ) {
  buf[bi] = path[pi];
  bi++ ; pi++ ;
  buf[bi] = '\0';
  if ( path[pi] == '\0' || path[pi] == '/' ) {
   mkdir(buf, 0700);
  }
 }
}

/* Create directory descriptions */

static void create_e17_menu_directory(const char *directory, const char *name)
{
/* char dot_directory[PATH_MAX+1]; */
 make_directory_path(directory);
}

/* Create the directory and fill it */

static void make_dir(struct group *root, int level, const char *dir_name, const char *use_term)
{
 struct item *item;
 char c_tmp[PATH_MAX];
 if (root == (struct group *)NULL)
  return;
 item = root->items;
 while (item->type != 0) {
  if (item->type == ITEM_MENU) {
   struct group *tmp;
   tmp = (struct group *)item->data;
   snprintf(c_tmp, PATH_MAX, "%s/%s", single_string(dir_name), single_string(tmp->name) );
   create_e17_menu_directory(c_tmp, tmp->name);
  } else if (item->type == ITEM_APP) {
   struct package *app;
   app = (struct package *)item->data;
   if (app->name && app->exec) {
    snprintf(c_tmp, PATH_MAX, "%s/%s.desktop", single_string(dir_name), single_string(app->name) );
    create_desktop_entry(c_tmp, app, use_term);
   }
  }
  item++;
 }
 item = root->items;
 while (item->type != 0) {
  if (item->type == ITEM_MENU) {
   struct group *tmp;
   tmp = (struct group *)item->data;
   snprintf(c_tmp, PATH_MAX, "%s/%s", dir_name, tmp->name);
   make_dir(item->data, level+1, c_tmp, use_term);
  }
  item++;
 }
}

/* Remove any previous menu entries */

static void clean_dir(const char *dir_name)
{
 struct dirent **namelist;
 int nr_files;
 int i;
 if (dir_name == NULL)
  return;
 nr_files = scandir(dir_name, &namelist, NULL, alphasort);
 for (i = 0; i < nr_files; i++) {
  if ( (strcmp(namelist[i]->d_name, ".") != 0) &&
  (strcmp(namelist[i]->d_name, "..") != 0) ) {
   struct stat st;
   char tmp[PATH_MAX];
   int retval;
   snprintf(tmp, PATH_MAX, "%s/%s", dir_name, namelist[i]->d_name);
   if (stat(tmp, &st) != 0) {
    fprintf(stderr, "Could not stat %s\n", tmp);
    return;
   }
   if (S_ISDIR(st.st_mode)) {
    clean_dir(tmp);
    retval = rmdir(tmp);
   } else {
    retval = unlink(tmp);
   }
   if (retval != 0) {
    perror("Could not remove");
    return;
   }
  }
  free(namelist[i]);
 }
 if (nr_files && namelist)
  free(namelist);	    
}

/* Final output function used by wmconfig.c */

void output_e17(struct group *root, const char *use_term)
{
 struct stat st;
 char root_dir[PATH_MAX];
 struct passwd *pw;
 setpwent();
 pw = getpwuid(getuid());
 if (pw == (struct passwd *)NULL) {
  fprintf(stderr, "Could not find out who you are (getpwnam failed)!\n");
  return;
 }
 endpwent();

/* Check if the outputdir option is given */

 if (output_directory == NULL) {
  snprintf(root_dir, PATH_MAX, "%s/%s", pw->pw_dir, E17_DIR);
 } else {
  snprintf(root_dir, PATH_MAX, "%s", output_directory);
 }

/* Final checks and menu creation */

 if (stat(root_dir, &st) == 0) {
  if (!S_ISDIR(st.st_mode)) {
   fprintf(stderr, "Error: %s exists but it is not a directory\n",
   root_dir);
   return;
  }
  clean_dir(root_dir);
  rmdir(root_dir);
 }
 create_e17_menu_directory(root_dir, DEFAULT_ROOT_NAME);
 make_dir(root, 0, root_dir, use_term);
}
